use strict;
use warnings;

package Bot::Net::Script::Help;
use base qw/ App::CLI::Command::Help /;

=head1 NAME

Bot::Net::Script::Help - Get help using Bot::Net

=head1 SYNOPSIS

Try:

  bin/botnet help <command>

=head1 AUTHORS

Andrew Sterling Hanenkamp C<< <hanenkamp@cpan.org> >>

=head1 COPYRIGHT AND LICENSE

Copyright 2007 Boomer Consulting, Inc. All Rights Reserved.

This program is free software and may be modified and redistributed under the same terms as Perl itself.

=cut

1;
